import dotenv from 'dotenv';
import { filter, Observable, Observer, Subscription } from 'rxjs';
import { v4 as uuidv4 } from 'uuid'
import ConsoleLogger from '../utils/log.utils';
import { WrappedMessage } from '../utils/message.ordering';
import { AdapterBase } from '../base/adapter.base';
import { FisMessage, GeneralEvent, TransportType, TransportMessage, TransportServiceInterface, TransmissionRole } from '../interface/interface';

dotenv.config();

export class ReceiverAdapter<T> extends AdapterBase<T> {

    constructor(clientId: string, adapterType: TransportType, transportService: TransportServiceInterface, role: TransmissionRole) {
        super(clientId, adapterType, transportService, role)
        this.console = new ConsoleLogger(`${this.adapterProfile.transportType}ReceiverAdapter`, ['adapter'])

        this.console.log({ message: `Contructing ReceiverAdapter for clientId: ${this.adapterProfile.clientId}` })
    }

    subscribeForIncoming(): Observable<GeneralEvent<any>> {
        this.console.log({ message: `Getting message bus for this connector: ${this.adapterProfile.clientId}` })
        return new Observable((observable: Observer<GeneralEvent<any>>) => {
            const subscription: Subscription = this.adapterProfile.transportService.subscribeForEvent().pipe(
                filter(event => event.type === `Transport Event`),
                filter((message: GeneralEvent<TransportMessage>) => (message.data as TransportMessage).source === this.adapterProfile.clientId),
                filter((message: GeneralEvent<any>) => message.event === 'New Message'),
            ).subscribe((message: GeneralEvent<TransportMessage>) => {
                this.console.log({ message: `Received ${(((message.data as TransportMessage).payload as WrappedMessage).payload as FisMessage).header.messageID} from ${((message.data as TransportMessage).source)}`, details: message })
                observable.next({
                    id: uuidv4(),
                    type: `Adapter Event`,
                    event: `New Message`,
                    date: new Date(),
                    data: message.data,
                    transport: undefined,
                    transportServiceId: null
                } as GeneralEvent<TransportMessage>);
            });

            // Clean up on unsubscription
            return () => {
                subscription.unsubscribe();
            };

        });
    }

}


