/* This is the message producer class. Just a standard class to generate mock up data to emulate real time messages on the go.
This is currently being used only in test cases for generating fake data to test the transmisison. Can configure the param to test
the transmission speed and so on. */
import { interval, map, Observable, Observer, Subject, take } from "rxjs"
import { FisMessage } from "../interface/interface"
import { v4 as uuidv4 } from 'uuid'
import ConsoleLogger from "../utils/log.utils"

//  This class is to emulate a message source that will be tranmitted to the target. Can be used to test request response as well
export class MessageProducer {
    private console = new ConsoleLogger('Message Producer', ['base'])
    private generalNotification: Subject<FisMessage> = new Subject()
    private incomingMessageBus!: Subject<FisMessage>
    private outgoingMessageBus: Subject<FisMessage> = new Subject()

    constructor(incomingMessageBus: Subject<FisMessage>) {
        this.console.log({ message: `Constructing Message Producer` })
        this.incomingMessageBus = incomingMessageBus

        this.generateNotifcation().subscribe(this.generalNotification)
        this.handleIncomingRequests(this.incomingMessageBus.asObservable(), this.outgoingMessageBus)
    }

    public getNotificationMessage(): Observable<FisMessage> {
        return this.generalNotification.asObservable()
    }

    public getOutgoingMessages(): Observable<FisMessage> {
        return this.outgoingMessageBus.asObservable()
    }

    // this is called no problem
    private handleIncomingRequests(requests: Observable<FisMessage>, outgoingMessageBus: Subject<FisMessage>, amount?: number): void {
        requests.subscribe((request: FisMessage) => {
            this.console.log({ message: `Generating response for new request ${request.header.messageID}` })
            this.generateMessage(request.header.messageID, amount ?? 10).subscribe({
                next: message => outgoingMessageBus.next(message),
                error: error => this.console.log({ message: 'observer Error', details: error }),
                complete: () => {
                    outgoingMessageBus.next({
                        header: {
                            messageID: request.header.messageID,
                            messageName: `ResponseMessage`
                        },
                        data: `Complete`
                    } as FisMessage)
                }
            })
        })
    }

    private generateMessage(requestID: string, amount: number): Observable<FisMessage> {
        return new Observable((response: Observer<FisMessage>) => {
            const intervalMessageGeneration = interval(1000).pipe(
                take(amount), // Ensures only 'amount' messages are generated
                map(() => {
                    const message: FisMessage = {
                        header: {
                            messageID: requestID,
                            messageName: 'ResponseMessage'
                        },
                        data: `Data`
                    };
                    return message;
                })
            );

            const subscription = intervalMessageGeneration.subscribe({
                next: message => response.next(message),
                error: error => response.error(error),
                complete: () => {
                    response.next({
                        header: {
                            messageID: requestID,
                            messageName: 'ResponseMessage'
                        },
                        data: `Complete`
                    });
                    response.complete();
                }
            });

            // Ensure cleanup on unsubscribe
            return () => subscription.unsubscribe();
        });
    }

    private generateNotifcation(): Observable<FisMessage> {
        return new Observable((response: Observer<FisMessage>) => {
            const intervalMessageGeneration = interval(1000).pipe(
                map(() => {
                    const message: FisMessage = {
                        header: {
                            messageID: uuidv4(),
                            messageName: 'NotificationMessage'
                        },
                        data: `Data`
                    };
                    return message;
                })
            );

            const subscription = intervalMessageGeneration.subscribe({
                next: message => response.next(message),
                error: error => response.error(error),
                complete: () => {
                    response.next({
                        header: {
                            messageID: uuidv4(),
                            messageName: 'NotificationMessage'
                        },
                        data: `Complete`
                    });
                    response.complete();
                }
            });

            // Ensure cleanup on unsubscribe
            return () => subscription.unsubscribe();
        });
    }
}
