import { Subject } from "rxjs";
import { GeneralEvent, TransportType, TransportServiceInterface, TransportSet } from "../interface/interface";
import { WebsocketTransportService } from "../transport/websocket";
import { HttpTransportService } from "../transport/http";
import ConsoleLogger from "./log.utils";
const console: ConsoleLogger = new ConsoleLogger(`TransportUtils`, ['transport'])
// Server to be set up as well as acquiring client information if needed. Like in the case for grpc and socket. Http not requ`ired.
export function setUpTransportService(transportSet: TransportSet, event: Subject<GeneralEvent<any>>, transportServiceArray: TransportServiceInterface[], url?: string): void {
    try {
        let transportService: TransportServiceInterface = instantiateTransportService(transportSet.transport, event)
        transportServiceArray.push(transportService)
        if (transportService instanceof WebsocketTransportService) {
            console.log({ message: `${url}` })
            console.log({ message: `Just Double Checking... this is websocket ${url ? `client` : `server`}` })
            if (url) {
                transportService.startClient(url)
            } else {
                transportService.startServer(transportSet.port);
            }
        } else if (transportService instanceof HttpTransportService) {
            console.log({ message: `Just Double Checking... this is http ${url ? `client` : `server`}` })
            // Additional Http-specific setup if needed.
            if (url) {
                transportService.startClient(url)
            } else {
                transportService.startServer(transportSet.port)
            }
        }
    } catch (error) {
        console.error({ message: 'Fail to set transport. Error in setting up transport', details: error })
    }
}

function instantiateTransportService(transportType: TransportType, event: Subject<GeneralEvent<any>>): TransportServiceInterface {
    if (transportType === 'Websocket') {
        return new WebsocketTransportService(event)
    }
    else if (transportType === 'Http') {
        return new HttpTransportService(event)
    } else {
        throw new Error(`No Transport Service Instantiated`)
    }
}

export function sortTransportFromEnv(transportSet: TransportSet[]): void {
    let transportList: string[] = process.env.Transport?.split(',') || []
    let portList: number[] = (process.env.PORT?.split(',') || []).map(port => Number(port));
    transportList.forEach((transport, index) => {
        transportSet.push({ transport: transport, port: portList[index] } as unknown as TransportSet)
    })
    console.log({ message: 'TransportSetList', details: transportSet })
}

